#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Written by Lucas Sinclair.

JRC Biomass Project.
Unit D1 Bioeconomy.

Typically you can use this class this like:

    >>> from waste_flow.treatment import waste_trt
    >>> print(waste_trt.df)
"""

# Built-in modules #

# Internal modules #
from waste_flow.zip_files import waste_trt as orig_trt

# First party modules #

# Third party modules #

###############################################################################
class WasteTreatment:

    @property
    def df(self):
        """Format and filter the dataframe."""
        # Load #
        df = orig_trt.df.copy()
        # Return #
        return df

###############################################################################
# Create singleton #
waste_trt = WasteTreatment()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Written by Lucas Sinclair.

JRC Biomass Project.
Unit D1 Bioeconomy.
"""

# Special variables #
__version__ = '1.0.9'

# Built-in modules #
import os, sys

# First party modules #
from autopaths    import Path
from plumbing.git import GitRepo

# Constants #
project_name  = 'waste_flow'
project_url   = 'https://github.com/xapple/waste_flow'

# Get paths to module #
self       = sys.modules[__name__]
module_dir = Path(os.path.dirname(self.__file__))

# The repository directory #
repos_dir = module_dir.directory

# The module is maybe in a git repository #
git_repo = GitRepo(repos_dir, empty=True)

# Determine where to cache things #
env_var_name = "WASTE_FLOW_CACHE"

# If it is specified by user #
if env_var_name in os.environ:
    cache_dir = os.environ[env_var_name]
    if not cache_dir.endswith('/'): cache_dir += '/'

# If it is not specified by user #
else:
    import tempfile
    cache_dir = tempfile.gettempdir() + '/waste_flow/'
    import warnings
    message = ("\n\n The cache location for waste_flow's data is not defined in"
               " the '%s' environment variable.\n In this case it will default"
               " to:\n\n '%s',\n which might lead to re-caching after every startup.\n")
    message = message % (env_var_name, cache_dir)
    warnings.warn(message)

# Monkey patch pandas library #
import plumbing.pandas_patching


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Written by Lucas Sinclair.

JRC Biomass Project.
Unit D1 Bioeconomy.
"""

# Built-in modules #

# Internal modules #
from waste_flow import module_dir

# First party modules #

# Third party modules #
import pandas

###############################################################################
# Load names #
nace_names  = module_dir + 'extra_data/nace_to_full_name.csv'
nace_names  = pandas.read_csv(str(nace_names))
waste_names = module_dir + 'extra_data/waste_to_full_name.csv'
waste_names = pandas.read_csv(str(waste_names))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Written by Lucas Sinclair.

JRC Biomass Project.
Unit D1 Bioeconomy.
"""

# Built-in modules #

# Internal modules #
from waste_flow.reports.base_template import ReportTemplate
from waste_flow                       import cache_dir
from waste_flow.reports.template      import Header, Footer

# First party modules #
from plumbing.cache    import property_cached
from pymarktex         import Document
from pymarktex.figures import ScaledFigure, BareFigure
from autopaths         import Path

# Third party modules #

###############################################################################
class ComparisonReport(Document):
    """
    A report generated in PDF describing several countries.
    """

    header_template = Header
    footer_template = Footer

    def __init__(self, output_path=None):
        # Paths #
        if output_path is None:
            self.output_path = Path(cache_dir + 'reports/comparison.pdf')
        else:
            self.output_path = Path(output_path)

    @property_cached
    def template(self): return ComparisonTemplate(self)

    def load_markdown(self): self.markdown = str(self.template)

###############################################################################
class ComparisonTemplate(ReportTemplate):
    """All the parameters to be rendered in the markdown template."""

    delimiters = (u'{{', u'}}')

    def __repr__(self):
        return '<%s object on %s>' % (self.__class__.__name__, self.parent)

    def __init__(self, parent):
        self.parent = parent
        self.report = parent

    #----------------------------- By country --------------------------------#
    def waste_gen_by_country(self):
        # Import #
        from waste_flow.viz.gen_by_country import countries, legend
        # Caption #
        caption = "Legend for waste generation"
        # Initialize #
        result = ""
        # Add the legend #
        result += str(ScaledFigure(graph   = legend,
                                   caption = caption,
                                   label   = 'wastes_legend',
                                   width   = '14em'))
        # Loop every country batch #
        for viz in countries.values():
            result += '\n--\n**%s**\n--\n\n' % viz.country.long_name
            for graph in viz.all_graphs:
                result += str(BareFigure(graph=graph)) + '\n\n'
        # Return #
        return result

    #----------------------------- By sector --------------------------------#
    def waste_gen_by_sector(self):
        # Import #
        from waste_flow.viz.gen_by_sector import sectors, legend
        from waste_flow.common            import nace_names
        # Caption #
        caption = "Legend for waste generation"
        # Initialize #
        result = ""
        # Add the legend #
        result += str(ScaledFigure(graph   = legend,
                                   caption = caption,
                                   label   = 'wastes_legend',
                                   width   = '14em'))
        # Loop every country batch #
        for viz in sectors.values():
            row  = nace_names.query('nace_r2 == @viz.sector')
            text = row.iloc[0]['description']
            result += '\n--\n**%s**\n--\n\n' % text
            for graph in viz.all_graphs:
                result += str(BareFigure(graph=graph)) + '\n\n'
        # Return #
        return result

###############################################################################
# Create singleton #
report = ComparisonReport()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Written by Lucas Sinclair.

JRC Biomass Project.
Unit D1 Bioeconomy.
"""

# Built-in modules #

# Internal modules #
from waste_flow import cache_dir
from waste_flow.country import all_countries
from waste_flow.common import nace_names

# First party modules #
from plumbing.graphs.multiplot import Multiplot
from plumbing.cache import property_cached
from plumbing.graphs.solo_legend import SoloLegend

# Third party modules #
from matplotlib import pyplot

# Where to store the graphs #
base_dir = cache_dir + 'graphs/gen_by_country/'

###############################################################################
class GenByCountryViz:
    """
    Every country has one GenByCountryViz object containing several graphs.
    Each graph contains a fixed number of subplots.
    Each subplot corresponds to one sector (nace).
    """

    def __init__(self, country):
        self.country = country

    def plot(self, *ag, **kw):
        return [graph.plot(*ag, **kw) for graph in self.all_graphs]

    @property_cached
    def df(self):
        # Load #
        df = self.country.dry_mass
        # Swap index levels #
        df = df.reset_index()
        df = df.sort_values(['nace_r2', 'year'])
        df = df.set_index(['nace_r2', 'year'])
        # Return #
        return df

    @property_cached
    def all_graphs(self):
        # Get all sectors #
        sectors = self.df.index.levels[0]
        # Sort sectors into batches of a given size #
        size    = GenCountryPlot.n_cols
        count   = len(sectors)
        batches = [sectors[i:i + size] for i in range(0, count, size)]
        # One graph per sector #
        result = [GenCountryPlot(self, batch) for batch in batches]
        # Return #
        return result

###############################################################################
class GenCountryPlot(Multiplot):

    # Basic params #
    formats    = ('pdf',)
    share_y    = False
    share_x    = False
    height     = 7
    width      = 30

    # Labels for axes #
    label_x = 'Year'
    label_y = 'Dry mass in kilograms'

    # Size of grid #
    n_rows = 1
    n_cols = 4

    def __init__(self, parent, batch):
        # Save batch #
        self.batch = batch
        # Call parent class #
        super(GenCountryPlot, self).__init__(parent, base_dir)

    @property_cached
    def short_name(self):
        return self.parent.country.code + '|' + '+'.join(c for c in self.batch)

    def lines_plot(self, sector, axes, **kw):
        # Load #
        df = self.parent.df
        # Filter for the current sector #
        df = df.loc[[sector]]
        # We don't need the sector column anymore #
        df = df.droplevel(0)
        # Get all waste types #
        wastes = df.columns
        # Plot every line #
        for waste in wastes:
            axes.plot(df[waste],
                      marker     = ".",
                      markersize = 20.0,
                      linewidth  = 5.0,
                      color      = legend.name_to_color[waste],
                      **kw)

    def plot(self, **kwargs):
        # Plot every country #
        for sector, axes in zip(self.batch, self.axes):
            self.lines_plot(sector, axes)

        # Change the X labels #
        self.set_x_labels(self.label_x)

        # Change the Y labels only for the left most graph #
        self.axes[0].set_ylabel(self.label_y, fontsize=12)

        # Remove ugly box around figures #
        self.remove_frame()

        # Adjust details on the subplots #
        self.y_grid_on()

        # Add the sector name as a title #
        for sector, axes in zip(self.batch, self.axes):
            row  = nace_names.query('nace_r2 == @sector')
            text = row.iloc[0]['description']
            if len(text) > 30: text = text[:30] + ' [...]'
            axes.text(0.05, 1.05, text, transform=axes.transAxes, ha="left", size=22)

        # Prune graphs if we are shorter than n_cols #
        if len(self.batch) < self.n_cols:
            for axes in self.axes[len(self.batch):]:
                self.hide_full_axes(axes)

        # Leave some space around the graph #
        pyplot.subplots_adjust(wspace=0.2, top=0.9, left=0.04, right=0.95, bottom=0.1)
        # Save #
        self.save_plot(**kwargs)
        # Convenience: return for display in notebooks for instance #
        return self.fig

###############################################################################
class GenByCountryLegend(SoloLegend):

    # Params #
    capitalize = False

    @property_cached
    def label_to_color(self):
        """Mapping of each waste to type to colors."""
        return {
            "Paper and cardboard wastes":     'gray',
            "Rubber wastes":                  'blue',
            "Wood wastes":                    'brown',
            "Textile wastes":                 'cyan',
            "Animal and mixed food waste":    'pink',
            "Vegetal wastes":                 'green',
            "Animal feces urine and manure":  'yellow',
        }

    @property_cached
    def name_to_color(self):
        return {
            'W072': 'gray',
            'W073': 'blue',
            'W075': 'brown',
            'W076': 'cyan',
            'W091': 'pink',
            'W092': 'green',
            'W093': 'yellow',
        }

###############################################################################
# Every country has a several graphs (each graph has several subplots) #
countries = {c.code: GenByCountryViz(c) for c in all_countries}

# Create a separate standalone legend #
legend = GenByCountryLegend(base_dir = base_dir)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Written by Lucas Sinclair.

JRC Biomass Project.
Unit D1 Bioeconomy.

Typically you can use this class this like:

    >>> from waste_flow.outputs import outputs
    >>> print(outputs.make_dry_mass())
"""

# Built-in modules #

# Internal modules #

# First party modules #
from plumbing.xls_tables import MultiDataFrameXLS

# Third party modules #

###############################################################################
class Outputs:
    """Takes care of creating some files that contain outputs."""

    def make_dry_mass(self, path):
        """
        Create an excel file with one sheet per country.
        And containing the dry mass tables for each year.
        """
        # Load #
        from waste_flow.generation import waste_gen
        df = waste_gen.dry_mass
        # Remove index #
        df = df.reset_index()
        # Group per country #
        groups = df.groupby(['country'])
        # Write each country #
        sheet_to_dfs = {k: self.one_country(k,v) for k,v in groups}
        # Save #
        multi_xls = MultiDataFrameXLS(sheet_to_dfs, path)
        # Do it #
        return multi_xls()

    def one_country(self, code, df):
        """
        Create an excel sheet with every year that a
        given country has.
        """
        # Drop the country column #
        df = df.drop('country', axis=1)
        # Group by year #
        groups = df.groupby('year')
        # Initialize #
        all_dfs = []
        # Loop #
        for year, table in groups:
            # Drop the year column #
            table = table.drop('year', axis=1)
            table = table.set_index('nace_r2')
            # Main title #
            title = "YEAR %s -- dry mass in kg -- W101 is spread out" % year
            # Other labels #
            sheet = {
                'dataframe': table,
                'title':     title,
                'x_title':   "Waste",
                'y_title':   "Activity",
                'x_label':   None,
                'y_label':   None,
                'x_extra':   None,
                'y_extra':   None,
            }
            # Append #
            all_dfs.append(sheet)
        # Return #
        return all_dfs

###############################################################################
# Create singleton #
outputs = Outputs()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Written by Lucas Sinclair.

JRC Biomass Project.
Unit D1 Bioeconomy.
"""

# Built-in modules #

# Internal modules #
from waste_flow.zip_files import waste_gen as gen_orig
from waste_flow import module_dir

# First party modules #
from plumbing.cache import property_cached

# Third party modules #
import pandas

# Load country codes #
country_codes = module_dir + 'extra_data/foastat_countries.csv'
country_codes = pandas.read_csv(str(country_codes))

###############################################################################
class Country:
    """Represents one country's dataset."""

    def __init__(self, code):
        # The reference country code #
        self.code = code

    def __repr__(self):
        return '%s object code "%s"' % (self.__class__, self.code)

    @property_cached
    def long_name(self):
        """The long name of this country."""
        row = country_codes.query('nuts_zero_2010 == @self.code')
        return row.iloc[0]['country']

    @property_cached
    def dry_mass(self):
        """
        Return rows that concern this country only for the
        waste generation dry mass table.
        """
        # Import #
        from waste_flow.generation import waste_gen
        # Load #
        df = waste_gen.dry_mass
        # Select rows for current country #
        df = df.loc[[self.code]].copy()
        # We don't need the country column anymore #
        df = df.droplevel(0)
        # Return #
        return df

###############################################################################
# Get all possible countries #
all_codes = list(gen_orig.df.country.unique())

# Create every country object #
all_countries = [Country(code) for code in all_codes]
countries     = {c.code: c for c in all_countries}



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Written by Lucas Sinclair.

JRC Biomass Project.
Unit D1 Bioeconomy.
"""

# Built-in modules #
import socket

# Internal modules #
import waste_flow

# First party modules #
from pymarktex.templates import Template
from plumbing.common     import pretty_now

###############################################################################
class ReportTemplate(Template):
    """Things that are common to most reports in waste_flow."""

    # Process info #
    def project_name(self):      return waste_flow.project_name
    def project_url(self):       return waste_flow.project_url
    def project_version(self):   return waste_flow.__version__
    def now(self):               return pretty_now()
    def hostname(self):          return socket.gethostname()
    def git(self):
        if not waste_flow.git_repo: return False
        return {'git_hash'  : waste_flow.git_repo.hash}



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Written by Lucas Sinclair.

JRC Biomass Project.
Unit D1 Bioeconomy.
"""

# Built-in modules #

# Internal modules #
from waste_flow                    import cache_dir
from waste_flow.common             import nace_names
from waste_flow.country            import countries
from waste_flow.viz.gen_by_country import legend
from waste_flow.generation         import waste_gen

# First party modules #
from plumbing.graphs.multiplot import Multiplot
from plumbing.cache import property_cached

# Third party modules #
from matplotlib import pyplot

# Where to store the graphs #
base_dir = cache_dir + 'graphs/gen_by_sector/'

###############################################################################
class GenBySectorViz:
    """
    Every sector has one GenBySectorViz object containing several graphs.
    Each graph contains a fixed number of subplots.
    Each subplot corresponds to one country.
    """

    def __init__(self, sector):
        self.sector = sector

    def plot(self, *ag, **kw):
        return [graph.plot(*ag, **kw) for graph in self.all_graphs]

    @property_cached
    def df(self):
        # Load #
        df = waste_gen.dry_mass
        # Reset index #
        df = df.reset_index()
        # Filter #
        df = df.query('nace_r2 == @self.sector')
        df = df.drop(columns={'nace_r2'})
        # Swap index levels #
        df = df.sort_values(['country', 'year'])
        df = df.set_index(['country', 'year'])
        # Return #
        return df

    @property_cached
    def all_graphs(self):
        # Get all countries #
        countries = self.df.index.levels[0]
        # Sort sectors into batches of a given size #
        size    = GenSectorPlot.n_cols
        count   = len(countries)
        batches = [countries[i:i + size] for i in range(0, count, size)]
        # One graph per sector #
        result = [GenSectorPlot(self, batch) for batch in batches]
        # Return #
        return result

###############################################################################
class GenSectorPlot(Multiplot):

    # Basic params #
    formats    = ('pdf',)
    share_y    = False
    share_x    = False
    height     = 7
    width      = 30

    # Labels for axes #
    label_x = 'Year'
    label_y = 'Dry mass in kilograms'

    # Size of grid #
    n_rows = 1
    n_cols = 4

    def __init__(self, parent, batch):
        # Save batch #
        self.batch = batch
        # Call parent class #
        super(GenSectorPlot, self).__init__(parent, base_dir)

    @property_cached
    def short_name(self):
        return self.parent.sector + '|' + '+'.join(s for s in self.batch)

    def lines_plot(self, country, axes, **kw):
        # Load #
        df = self.parent.df
        # Filter for the current country #
        df = df.loc[[country]]
        # We don't need the country column anymore #
        df = df.droplevel(0)
        # Get all waste types #
        wastes = df.columns
        # Plot every line #
        for waste in wastes:
            axes.plot(df[waste],
                      marker     = ".",
                      markersize = 20.0,
                      linewidth  = 5.0,
                      color      = legend.name_to_color[waste],
                      **kw)

    def plot(self, **kwargs):
        # Plot every country #
        for country, axes in zip(self.batch, self.axes):
            self.lines_plot(country, axes)

        # Change the X labels #
        self.set_x_labels(self.label_x)

        # Change the Y labels only for the left most graph #
        self.axes[0].set_ylabel(self.label_y, fontsize=12)

        # Remove ugly box around figures #
        self.remove_frame()

        # Adjust details on the subplots #
        self.y_grid_on()

        # Add the country name as a title #
        for country, axes in zip(self.batch, self.axes):
            text = countries[country].long_name
            axes.text(0.05, 1.05, text, transform=axes.transAxes, ha="left", size=22)

        # Prune graphs if we are shorter than n_cols #
        if len(self.batch) < self.n_cols:
            for axes in self.axes[len(self.batch):]:
                self.hide_full_axes(axes)

        # Leave some space around the graph #
        pyplot.subplots_adjust(wspace=0.2, top=0.9, left=0.04, right=0.95, bottom=0.1)
        # Save #
        self.save_plot(**kwargs)
        # Convenience: return for display in notebooks for instance #
        return self.fig

###############################################################################
# Every sector has a several graphs (each graph has several subplots) #
sectors = {s: GenBySectorViz(s) for s in nace_names['nace_r2']}


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Written by Lucas Sinclair.

JRC Biomass Project.
Unit D1 Bioeconomy.
"""

# Built-in modules #
import os, inspect

# Internal modules #
from pymarktex.templates import Template

# First party modules #
from autopaths import Path

# Get current directory #
file_name = os.path.abspath((inspect.stack()[0])[1])
this_dir  = Path(os.path.dirname(os.path.abspath(file_name)) + '/')

###############################################################################
class Header(Template):
    """All the parameters to be rendered in the LaTeX header template"""
    def image_path(self): return (this_dir + 'logo.png').unix_style

###############################################################################
class Footer(Template):
    """All the parameters to be rendered in the LaTeX footer template"""
    pass


from setuptools import setup, find_packages

setup(
        name             = 'waste_flow',
        version          = '1.0.9',
        description      = 'A package for retrieving data concerning waste management on the European continent.',
        long_description = open('README.md').read(),
        long_description_content_type = 'text/markdown',
        license          = 'MIT',
        url              = 'http://github.com/xapple/waste_flow/',
        author           = 'Lucas Sinclair',
        author_email     = 'lucas.sinclair@me.com',
        packages         = find_packages(),
        install_requires = ['pandas>=1.0.0', 'requests', 'numpy>=1.16', 'matplotlib',
                            'autopaths>=1.3.6', 'plumbing>=2.6.7', 'pymarktex>=1.3.2'],
        include_package_data = True,
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Written by Lucas Sinclair.

JRC Biomass Project.
Unit D1 Bioeconomy.

Typically you can use this class this like:

    >>> from waste_flow.zip_files import waste_gen, waste_trt
    >>> print(waste_gen.cache_is_valid)
    >>> print(waste_trt.cache_is_valid)

To download the zips:

    >>> from waste_flow.zip_files import waste_gen, waste_trt
    >>> print(waste_gen.refresh_cache())
    >>> print(waste_trt.refresh_cache())

To look at the raw contents as dataframes:

    >>> from waste_flow.zip_files import waste_gen, waste_trt
    >>> print(waste_gen.raw_csv)
    >>> print(waste_trt.raw_csv)

To look at the processed contents as dataframes:

    >>> from waste_flow.zip_files import waste_gen, waste_trt
    >>> print(waste_gen.processed_csv)
    >>> print(waste_trt.processed_csv)

To look at the filtered contents as dataframes:

    >>> from waste_flow.zip_files import waste_gen, waste_trt
    >>> print(waste_gen.df)
    >>> print(waste_trt.df)
"""

# Built-in modules #
import gzip, io

# Internal modules #
from waste_flow import cache_dir

# First party modules #
from plumbing.cache    import property_pickled_at
from plumbing.scraping import download_from_url
from autopaths         import Path

# Third party modules #
import pandas, numpy

###############################################################################
class ZipFile:
    """
    Download a zipped CSV file containing all countries and all categories
    from the EUROSTAT webserver.

    The original strategy to convert colons to NaN was the following:

        # If there is a colon in a cell, it's not a number #
        def colon_to_nan(cell): return numpy.NaN if ':' in cell else cell
        df = df.applymap(colon_to_nan)
    """

    def __init__(self, zip_cache_dir):
        # Record where the cache will be located on disk #
        self.cache_dir = zip_cache_dir
        # Where the file should be downloaded to #
        self.zip_path = Path(self.cache_dir + self.zip_name)

    def refresh_cache(self):
        """Will download the required zip files to the cache directory."""
        return download_from_url(self.base_url + self.url_param,
                                 self.zip_path,
                                 stream     = False,
                                 progress   = False,
                                 uncompress = False)

    # ---------------------------- Properties --------------------------------#
    @property
    def cache_is_valid(self, check_md5=True):
        """Checks if the file needed has been correctly downloaded."""
        # Simple case #
        if not self.zip_path.exists: return False
        if not check_md5:            return True
        # Check the MD5 #
        return self.zip_path.md5 == self.md5

    @property
    def raw_csv(self):
        """
        Loads the big CSV that's inside the ZIP into memory.
        We don't specify na_values = [':', ': '] here because it's done later.
        """
        # Check the cache #
        if not self.cache_is_valid:
            self.refresh_cache()
            if not self.cache_is_valid: raise Exception("Error refreshing cache")
        # Load the CSV #
        with gzip.open(self.zip_path) as csv_handle:
            text_mode = io.TextIOWrapper(csv_handle, encoding=self.encoding)
            return pandas.read_csv(text_mode, sep = ',|\t', engine = 'python')

    @property_pickled_at('csv_cache_path')
    def processed_csv(self):
        """Format the data frame and store it in cache."""
        # Load #
        df = self.raw_csv
        # Strange name due to wrong column parsing and tabs #
        df = df.rename(columns={'geo\\time': 'country'})
        # Also many column names have trailing whitespace #
        df.columns = df.columns.str.strip().str.replace(' ','_')
        # Some cells have a number and a character or just a character #
        for col in map(str, (2016, 2014, 2012, 2010, 2008, 2006, 2004)):
            df[col] = df[col].apply(pandas.to_numeric, errors = 'coerce')
        # Return #
        return df

    @property
    def df(self):
        """Filter the dataframe."""
        # Load #
        df = self.processed_csv
        # Filter for those that we want - UNIT #
        df = df.query("unit == 'T'")
        df = df.drop(columns=['unit'])
        # Filter for those that we want - HAZARD #
        df = df.query("hazard == 'HAZ_NHAZ'")
        df = df.drop(columns=['hazard'])
        # Albania, Kosovo and Turkey are missing entries, drop them #
        df = df.query("country != 'AL'")
        df = df.query("country != 'XK'")
        df = df.query("country != 'TR'")
        # Return #
        return df

    #--------------------------------- Cache ---------------------------------#
    @property
    def csv_cache_path(self):
        """Specify where on the file system we will pickle the property."""
        return self.cache_dir + self.short_name + '.pickle'

###############################################################################
class WasteGen(ZipFile):

    short_name = "env_wasgen"
    long_name  = "Generation of waste by waste category, hazardousness and NACE Rev. 2 activity."
    base_url   = "https://ec.europa.eu/eurostat/estat-navtree-portlet-prod/BulkDownloadListing"
    url_param  = "?file=data/env_wasgen.tsv.gz"
    zip_name   = "env_wasgen.tsv.gz"
    csv_name   = "env_wasgen.tsv"
    encoding   = "ISO-8859-1"
    md5        = "df2ab56bb48bb2bf0e3266d08fa4d408"

#-----------------------------------------------------------------------------#
class WasteTrt(ZipFile):

    short_name = "env_wastrt"
    long_name  = "Treatment of waste by waste category, hazardousness and waste management operations."
    base_url   = "https://ec.europa.eu/eurostat/estat-navtree-portlet-prod/BulkDownloadListing"
    url_param  = "?file=data/env_wastrt.tsv.gz"
    zip_name   = "env_wastrt.tsv.gz"
    csv_name   = "env_wastrt.tsv"
    encoding   = "ISO-8859-1"
    md5        = "bfbfc036921b22698e52eab168f3f891"

###############################################################################
# Create singletons #
waste_gen = WasteGen(cache_dir + 'eurostat/')
waste_trt = WasteTrt(cache_dir + 'eurostat/')


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Written by Lucas Sinclair.

JRC Biomass Project.
Unit D1 Bioeconomy.

Typically you can use this class this like:

    >>> from waste_flow.generation import waste_gen
    >>> print(waste_gen.dry_mass)
"""

# Built-in modules #

# Internal modules #
from waste_flow           import module_dir, cache_dir
from waste_flow.zip_files import waste_gen as orig_gen

# First party modules #
from plumbing.cache import property_pickled_at
from plumbing.cache import property_cached

# Third party modules #
import pandas

###############################################################################
class WasteGeneration:

    @property
    def filtered(self):
        """Filter the dataframe some more."""
        # Load #
        df = orig_gen.df
        # Filter for only the waste categories we are interested in #
        from waste_flow.mappings import wastes_selected
        df = df.query("waste in @wastes_selected")
        # Filter for only the nace categories we are interested in #
        from waste_flow.mappings import nace_selected
        df = df.query("nace_r2 in @nace_selected")
        # Return #
        return df

    @property
    def long_format(self):
        """
        Transform the dataframe into the long format
        (and not the wide format).
        """
        # Load #
        df = self.filtered
        # Get categorical columns #
        id_vars = [name for name, kind in df.dtypes.items() if kind == object]
        # Unpivot #
        df = df.melt(id_vars=id_vars, value_name='tonnes', var_name='year')
        # Sort by country then by year etc. #
        columns_order = ['country', 'year', 'nace_r2', 'waste']
        df = df[columns_order + ['tonnes']]
        df = df.sort_values(columns_order)
        # Return #
        return df

    @property
    def wide_format(self):
        """
        Transform the dataframe into the wide format
        (and not the long format).
        """
        # Load #
        df = self.long_format
        # Pivot #
        df = df.pipe(pandas.pivot_table,
                     index   = ['country', 'year', 'nace_r2'],
                     columns = ['waste'],
                     values  = ['tonnes'])
        # Return #
        return df

    @property_pickled_at('cache_path')
    def spread_muni(self):
        """
        Remove the municipal waste category and spread it (distribute it)
        in other categories according to specific breakdown proportions.
        These proportions are in "municipal_breakdown.csv"
        The municipal waste (W101) should be called as such because otherwise
        it can be confused with the "household" nace sector.
        """
        # Load municipal breakdown #
        muni_break = module_dir + 'extra_data/municipal_breakdown.csv'
        muni_break = pandas.read_csv(str(muni_break), index_col=0)
        muni_break = muni_break.fraction
        # Load dataframe #
        df = self.long_format
        # Group #
        groups = df.groupby(['country', 'year', 'nace_r2'])
        # Function #
        def remove_muni(group):
            # Index by waste type #
            pass
            # Pop a row (there is no pop function in pandas) #
            selector = group.waste == 'W101'
            muni_tot = group[selector]['tonnes'].iloc[0]
            result   = group[~selector]
            # Distribute the tonnes in the other categories #
            spread = muni_break * muni_tot
            # Pandas doesn't really have an add() that conserves NaNs #
            result = result.set_index('waste')
            rows   = set(spread.index).intersection(result.index)
            result.loc[rows, 'tonnes'] += spread
            # Don't return the grouping variables #
            return result['tonnes']
        # Apply function #
        df = groups.apply(remove_muni)
        # Return #
        return df

    @property_cached
    def dry_mass(self):
        """
        Convert all the 'tonnes' values to their dry weight equivalent
        in kilograms.
        """
        # Load dry wet coefficients #
        dry_coef = module_dir + 'extra_data/dry_weight_coef.csv'
        dry_coef = pandas.read_csv(str(dry_coef), index_col=0)
        dry_coef = 1 - dry_coef.fraction
        # Load dataframe #
        df = self.spread_muni.copy()
        # Multiply for dry mass #
        df *= dry_coef
        # Multiply for tonnes to kg #
        df *= 1000
        # Return #
        return df

    @property
    def dry_long(self):
        """Same as above but in the long format."""
        # Load #
        df = self.dry_mass
        df = df.reset_index()
        df = df.melt(id_vars=['country', 'year', 'nace_r2'], value_name='tonnes', var_name='waste')
        # Return #
        return df

    #--------------------------------- Cache ---------------------------------#
    @property
    def cache_path(self):
        """Specify where on the file system we will pickle the property."""
        return cache_dir + 'dataframes/' + 'generation.pickle'

###############################################################################
# Create singleton #
waste_gen = WasteGeneration()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Written by Lucas Sinclair.

JRC Biomass Project.
Unit D1 Bioeconomy.
"""

# Built-in modules #

# Internal modules #

# First party modules #

# Third party modules #

###############################################################################
# Wastes #
wastes_selected = ['W072', 'W073', 'W075', 'W076', 'W091', 'W092', 'W093', 'W101']

municipal = 'W101'

# Sectors #
industrial = ['A', 'B', 'C', 'D',
              'E36_E37_E39', 'E38',
              'F', 'G-U_X_G4677', 'G4677']

household  = ['EP_HH']

nace_selected = industrial + household